@@ -35,17 +35,19 @@ static GMainLoop *loop = NULL;
 static GTimer *log_timer;
 static int log_fd = -1;
 static gboolean debug = FALSE;
+static gboolean stopping = FALSE;
 
 static DisplayManager *display_manager = NULL;
 static XDMCPServer *xdmcp_server = NULL;
 static VNCServer *vnc_server = NULL;
+static guint bus_id = 0;
 static GDBusConnection *bus = NULL;
 static guint bus_id;
 static GDBusNodeInfo *seat_info;
-static GHashTable *seat_bus_entries;
+static GHashTable *seat_bus_entries = NULL;
 static guint seat_index = 0;
 static GDBusNodeInfo *session_info;
-static GHashTable *session_bus_entries;
+static GHashTable *session_bus_entries = NULL;
 static guint session_index = 0;
 static gint exit_code = EXIT_SUCCESS;
 
@@ -138,11 +140,34 @@ signal_cb (Process *process, int signum)
     // FIXME: Stop XDMCP server
 }
 
+static gboolean
+exit_cb (gpointer data)
+{
+    exit (exit_code);
+}
+
 static void
 display_manager_stopped_cb (DisplayManager *display_manager)
 {
     g_debug ("Stopping Light Display Manager");
-    exit (exit_code);
+
+    /* Mark as stopping because bus_acquired_cb can be called after this function, and even
+     * more weirdly display_manager is not NULL even though the following lines set it to
+     * be! */
+    stopping = TRUE;
+
+    /* Clean up display manager */
+    g_object_unref (display_manager);
+    display_manager = NULL;
+
+    /* Remove D-Bus interface */
+    g_bus_unown_name (bus_id);
+    if (seat_bus_entries)
+        g_hash_table_unref (seat_bus_entries);
+    if (session_bus_entries)
+        g_hash_table_unref (session_bus_entries);
+
+    g_idle_add (exit_cb, NULL);
 }
 
 static GVariant *
@@ -300,6 +325,7 @@ handle_display_manager_call (GDBusConnection       *connection,
         }
         else// FIXME: Need to make proper error
             g_dbus_method_invocation_return_error (invocation, G_DBUS_ERROR, G_DBUS_ERROR_FAILED, "Failed to start seat");
+        g_object_unref (seat);
     }
     else if (g_strcmp0 (method_name, "AddLocalXSeat") == 0)
     {
@@ -340,6 +366,7 @@ handle_display_manager_call (GDBusConnection       *connection,
         }
         else// FIXME: Need to make proper error
             g_dbus_method_invocation_return_error (invocation, G_DBUS_ERROR, G_DBUS_ERROR_FAILED, "Failed to start seat");
+        g_object_unref (seat);
     }
     /* NOTE: This method is deprecated, use the XSG_SEAT_PATH environment variable instead */
     else if (g_strcmp0 (method_name, "GetSeatForCookie") == 0)
@@ -733,8 +760,11 @@ bus_acquired_cb (GDBusConnection *connection,
         "</node>";
     GDBusNodeInfo *display_manager_info;
     GList *link;
-  
-    g_debug ("Acquired bus name");
+
+    if (stopping)
+        return;
+
+    g_debug ("Acquired bus name %s", name);
 
     bus = connection;
 
@@ -751,6 +781,7 @@ bus_acquired_cb (GDBusConnection *connection,
                                                 &display_manager_vtable,
                                                 NULL, NULL,
                                                 NULL);
+    g_dbus_node_info_unref  (display_manager_info);
 
     seat_bus_entries = g_hash_table_new_full (g_direct_hash, g_direct_equal, g_object_unref, bus_entry_free);
     session_bus_entries = g_hash_table_new_full (g_direct_hash, g_direct_equal, g_object_unref, bus_entry_free);
@@ -822,7 +853,8 @@ main (int argc, char **argv)
 {
     FILE *pid_file;
     GOptionContext *option_context;
-    gchar **groups, **i;
+    gboolean result;
+    gchar **groups, **i, *dir;
     gint n_seats = 0;
     gboolean explicit_config = FALSE;
     gboolean test_mode = FALSE;
@@ -903,22 +935,25 @@ main (int argc, char **argv)
 
     g_thread_init (NULL);
     g_type_init ();
+    loop = g_main_loop_new (NULL, FALSE);
 
     g_signal_connect (process_get_current (), "got-signal", G_CALLBACK (signal_cb), NULL);
 
     option_context = g_option_context_new (/* Arguments and description for --help test */
                                            _("- Display Manager"));
     g_option_context_add_main_entries (option_context, options, GETTEXT_PACKAGE);
-    if (!g_option_context_parse (option_context, &argc, &argv, &error))
+    result = g_option_context_parse (option_context, &argc, &argv, &error);
+    if (error)
+        g_printerr ("%s\n", error->message);
+    g_clear_error (&error);
+    g_option_context_free (option_context);
+    if (!result)
     {
-        if (error)
-            fprintf (stderr, "%s\n", error->message);
-        fprintf (stderr, /* Text printed out when an unknown command-line argument provided */
-                 _("Run '%s --help' to see a full list of available command line options."), argv[0]);
-        fprintf (stderr, "\n");
+        g_printerr (/* Text printed out when an unknown command-line argument provided */
+                    _("Run '%s --help' to see a full list of available command line options."), argv[0]);
+        g_printerr ("\n");
         return EXIT_FAILURE;
     }
-    g_clear_error (&error);
 
     if (show_version)
     {
@@ -1094,11 +1129,15 @@ main (int argc, char **argv)
     g_free (session_wrapper);
 
     /* Create run and cache directories */
-    g_mkdir_with_parents (config_get_string (config_get_instance (), "LightDM", "log-directory"), S_IRWXU | S_IXGRP | S_IXOTH);  
-    g_mkdir_with_parents (config_get_string (config_get_instance (), "LightDM", "run-directory"), S_IRWXU | S_IXGRP | S_IXOTH);
-    g_mkdir_with_parents (config_get_string (config_get_instance (), "LightDM", "cache-directory"), S_IRWXU | S_IXGRP | S_IXOTH);
-
-    loop = g_main_loop_new (NULL, FALSE);
+    dir = config_get_string (config_get_instance (), "LightDM", "log-directory");
+    g_mkdir_with_parents (dir, S_IRWXU | S_IXGRP | S_IXOTH);
+    g_free (dir);
+    dir = config_get_string (config_get_instance (), "LightDM", "run-directory");
+    g_mkdir_with_parents (dir, S_IRWXU | S_IXGRP | S_IXOTH);
+    g_free (dir);
+    dir = config_get_string (config_get_instance (), "LightDM", "cache-directory");
+    g_mkdir_with_parents (dir, S_IRWXU | S_IXGRP | S_IXOTH);
+    g_free (dir);
 
     log_init ();
 
@@ -1108,14 +1147,14 @@ main (int argc, char **argv)
     g_free (config_path);
 
     g_debug ("Using D-Bus name %s", LIGHTDM_BUS_NAME);
-    g_bus_own_name (getuid () == 0 ? G_BUS_TYPE_SYSTEM : G_BUS_TYPE_SESSION,
-                    LIGHTDM_BUS_NAME,
-                    G_BUS_NAME_OWNER_FLAGS_NONE,
-                    bus_acquired_cb,
-                    NULL,
-                    name_lost_cb,
-                    NULL,
-                    NULL);
+    bus_id = g_bus_own_name (getuid () == 0 ? G_BUS_TYPE_SYSTEM : G_BUS_TYPE_SESSION,
+                             LIGHTDM_BUS_NAME,
+                             G_BUS_NAME_OWNER_FLAGS_NONE,
+                             bus_acquired_cb,
+                             NULL,
+                             name_lost_cb,
+                             NULL,
+                             NULL);
 
     if (getuid () != 0)
         g_debug ("Running in user mode");
@@ -1152,6 +1191,7 @@ main (int argc, char **argv)
         {
             set_seat_properties (seat, config_section);
             display_manager_add_seat (display_manager, seat);
+            g_object_unref (seat);
             n_seats++;
         }
         else
@@ -1175,6 +1215,7 @@ main (int argc, char **argv)
             set_seat_properties (seat, NULL);
             seat_set_property (seat, "exit-on-failure", "true");
             display_manager_add_seat (display_manager, seat);
+            g_object_unref (seat);
         }
         else
             g_warning ("Failed to create default seat");
