@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2010 Robert Ancell.
+ * Copyright (C) 2010-2011 Robert Ancell.
  * Author: Robert Ancell <robert.ancell@canonical.com>
  * 
  * This program is free software: you can redistribute it and/or modify it under
@@ -20,7 +20,7 @@
 #include "xdmcp-server.h"
 #include "xdmcp-protocol.h"
 #include "xdmcp-session-private.h"
-#include "xauth.h"
+#include "xauthority.h"
 
 enum {
     NEW_SESSION,
@@ -42,19 +42,8 @@ struct XDMCPServerPrivate
     /* Status to report to clients */
     gchar *status;
 
-    /* Auhentication scheme to use */
-    gchar *authentication_name;
-
-    /* Auhentication data */  
-    guchar *authentication_data;
-    gsize authentication_data_length;
-
-    /* Authorization scheme to use */
-    gchar *authorization_name;
-
-    /* Authorization data */
-    guchar *authorization_data;
-    gsize authorization_data_length;
+    /* XDM-AUTHENTICATION-1 key */
+    gchar *key;
 
     /* Active XDMCP sessions */
     GHashTable *sessions;
@@ -71,18 +60,22 @@ xdmcp_server_new (void)
 void
 xdmcp_server_set_port (XDMCPServer *server, guint port)
 {
+    g_return_if_fail (server != NULL);
     server->priv->port = port;
 }
 
 guint
 xdmcp_server_get_port (XDMCPServer *server)
 {
+    g_return_val_if_fail (server != NULL, 0);
     return server->priv->port;
 }
 
 void
 xdmcp_server_set_hostname (XDMCPServer *server, const gchar *hostname)
 {
+    g_return_if_fail (server != NULL);
+
     g_free (server->priv->hostname);
     server->priv->hostname = g_strdup (hostname);
 }
@@ -90,12 +83,15 @@ xdmcp_server_set_hostname (XDMCPServer *server, const gchar *hostname)
 const gchar *
 xdmcp_server_get_hostname (XDMCPServer *server)
 {
+    g_return_val_if_fail (server != NULL, NULL);
     return server->priv->hostname;
 }
 
 void
 xdmcp_server_set_status (XDMCPServer *server, const gchar *status)
 {
+    g_return_if_fail (server != NULL);
+
     g_free (server->priv->status);
     server->priv->status = g_strdup (status);
 }
@@ -103,65 +99,16 @@ xdmcp_server_set_status (XDMCPServer *server, const gchar *status)
 const gchar *
 xdmcp_server_get_status (XDMCPServer *server)
 {
+    g_return_val_if_fail (server != NULL, NULL);
     return server->priv->status;
 }
 
 void
-xdmcp_server_set_authentication (XDMCPServer *server, const gchar *name, const guchar *data, gsize data_length)
+xdmcp_server_set_key (XDMCPServer *server, const gchar *key)
 {
-    g_free (server->priv->authentication_name);
-    server->priv->authentication_name = g_strdup (name);
-    g_free (server->priv->authentication_data);
-    server->priv->authentication_data = g_malloc (data_length);
-    server->priv->authentication_data_length = data_length;
-    memcpy (server->priv->authentication_data, data, data_length);
-}
-
-const gchar *
-xdmcp_server_get_authentication_name (XDMCPServer *server)
-{
-    return server->priv->authentication_name;
-}
-
-const guchar *
-xdmcp_server_get_authentication_data (XDMCPServer *server)
-{
-    return server->priv->authentication_data;
-}
-
-gsize
-xdmcp_server_get_authentication_data_length (XDMCPServer *server)
-{
-    return server->priv->authentication_data_length;
-}
-
-void
-xdmcp_server_set_authorization (XDMCPServer *server, const gchar *name, const guchar *data, gsize data_length)
-{
-    g_free (server->priv->authorization_name);
-    server->priv->authorization_name = g_strdup (name);
-    g_free (server->priv->authorization_data);
-    server->priv->authorization_data = g_malloc (data_length);
-    server->priv->authorization_data_length = data_length;
-    memcpy (server->priv->authorization_data, data, data_length);
-}
-
-const gchar *
-xdmcp_server_get_authorization_name (XDMCPServer *server)
-{
-    return server->priv->authorization_name;
-}
-
-const guchar *
-xdmcp_server_get_authorization_data (XDMCPServer *server)
-{
-    return server->priv->authorization_data;
-}
-
-gsize
-xdmcp_server_get_authorization_data_length (XDMCPServer *server)
-{
-    return server->priv->authorization_data_length;
+    g_return_if_fail (server != NULL);
+    g_free (server->priv->key);
+    server->priv->key = g_strdup (key);
 }
 
 static gboolean
@@ -200,7 +147,7 @@ get_session (XDMCPServer *server, guint16 id)
 static void
 send_packet (GSocket *socket, GSocketAddress *address, XDMCPPacket *packet)
 {
-    guchar data[1024];
+    guint8 data[1024];
     gssize n_written;
 
     g_debug ("Send %s", xdmcp_packet_tostring (packet));
@@ -212,37 +159,46 @@ send_packet (GSocket *socket, GSocketAddress *address, XDMCPPacket *packet)
     {
         GError *error = NULL;
 
-        if (g_socket_send_to (socket, address, (gchar *) data, n_written, NULL, &error) < 0)
+        g_socket_send_to (socket, address, (gchar *) data, n_written, NULL, &error);
+        if (error)
             g_warning ("Error sending packet: %s", error->message);
-
         g_clear_error (&error);
     }
 }
 
+static const gchar *
+get_authentication_name (XDMCPServer *server)
+{
+    if (server->priv->key)
+        return "XDM-AUTHENTICATION-1";
+    else
+        return "";
+}
+
 static void
 handle_query (XDMCPServer *server, GSocket *socket, GSocketAddress *address, XDMCPPacket *packet)
 {
     XDMCPPacket *response;
     gchar **i;
-    gboolean match_authentication = FALSE;
+    gchar *authentication_name = NULL;
 
     /* If no authentication requested and we are configured for none then allow */
-    if (packet->Query.authentication_names[0] == NULL && strcmp (server->priv->authentication_name, "") == 0)
-        match_authentication = TRUE;
+    if (packet->Query.authentication_names[0] == NULL && server->priv->key == NULL)
+        authentication_name = "";
 
     for (i = packet->Query.authentication_names; *i; i++)
     {
-        if (strcmp (*i, server->priv->authentication_name) == 0)
+        if (strcmp (*i, get_authentication_name (server)) == 0 && server->priv->key != NULL)
         {
-            match_authentication = TRUE;
+            authentication_name = *i;
             break;
         }
     }
 
-    if (match_authentication)
+    if (authentication_name)
     {
         response = xdmcp_packet_alloc (XDMCP_Willing);
-        response->Willing.authentication_name = g_strdup (server->priv->authentication_name);
+        response->Willing.authentication_name = g_strdup (authentication_name);
         response->Willing.hostname = g_strdup (server->priv->hostname);
         response->Willing.status = g_strdup (server->priv->status);
     }
@@ -250,8 +206,8 @@ handle_query (XDMCPServer *server, GSocket *socket, GSocketAddress *address, XDM
     {
         response = xdmcp_packet_alloc (XDMCP_Unwilling);
         response->Unwilling.hostname = g_strdup (server->priv->hostname);
-        if (strcmp (server->priv->authentication_name, "") != 0)
-            response->Unwilling.status = g_strdup_printf ("No matching authentication, server requires %s", server->priv->authentication_name);
+        if (server->priv->key)
+            response->Unwilling.status = g_strdup_printf ("No matching authentication, server requires %s", get_authentication_name (server));
         else
             response->Unwilling.status = g_strdup ("Server does not support authentication");
     }
@@ -261,31 +217,119 @@ handle_query (XDMCPServer *server, GSocket *socket, GSocketAddress *address, XDM
     xdmcp_packet_free (response);
 }
 
+static guint8
+atox (char c)
+{
+    if (c >= '0' && c <= '9')
+        return c - '0';
+    if (c >= 'a' && c <= 'f')
+        return c - 'a' + 10;
+    if (c >= 'A' && c <= 'F')
+        return c - 'A' + 10;
+    return 0;
+}
+
+static void
+decode_key (const gchar *key, guint8 *data)
+{
+    gint i;
+
+    memset (data, 0, sizeof (data));
+    if (strncmp (key, "0x", 2) == 0 || strncmp (key, "0X", 2) == 0)
+    {
+        for (i = 0; i < 8; i++)
+        {
+            if (key[i*2] == '\0')
+                break;
+            data[i] |= atox (key[i*2]) << 8;
+            if (key[i*2+1] == '\0')
+                break;
+            data[i] |= atox (key[i*2+1]);
+        }
+    }
+    else
+    {
+        for (i = 1; i < 8 && key[i-1]; i++)
+           data[i] = key[i-1];
+    }
+}
+
 static void
 handle_request (XDMCPServer *server, GSocket *socket, GSocketAddress *address, XDMCPPacket *packet)
 {
     int i;
     XDMCPPacket *response;
     XDMCPSession *session;
-    guchar *authentication_data = NULL;
+    guint8 *authentication_data = NULL;
     gsize authentication_data_length = 0;
     gboolean match_authorization = FALSE;
-    guchar *authorization_data = NULL;
+    gchar *authorization_name;
+    guint8 *authorization_data = NULL;
     gsize authorization_data_length = 0;
-    guchar *session_authorization_data = NULL;
+    guint8 *session_authorization_data = NULL;
     gsize session_authorization_data_length = 0;
     gchar **j;
-    GInetAddress *address4 = NULL, *address6 = NULL;
+    guint16 family;
+    GInetAddress *xserver_address = NULL;
+    gchar *display_number;
     XdmAuthKeyRec rho;
-  
-    /* Must be using our authentication scheme */
-    if (strcmp (packet->Request.authentication_name, server->priv->authentication_name) != 0)
+
+    /* Try and find an IPv6 address */
+    for (i = 0; i < packet->Request.n_connections; i++)
+    {
+        XDMCPConnection *connection = &packet->Request.connections[i];
+        if (connection->type == XAUTH_FAMILY_INTERNET6 && connection->address.length == 16)
+        {
+            family = connection->type;
+            xserver_address = g_inet_address_new_from_bytes (connection->address.data, G_SOCKET_FAMILY_IPV6);
+
+            /* We can't use link-local addresses, as we need to know what interface it is on */
+            if (g_inet_address_get_is_link_local (xserver_address))
+            {
+                g_object_unref (xserver_address);
+                xserver_address = NULL;
+            }
+            else
+                break;
+        }
+    }
+
+    /* If no IPv6 address, then try and find an IPv4 one */
+    if (!xserver_address)
+    {
+        for (i = 0; i < packet->Request.n_connections; i++)
+        {
+            XDMCPConnection *connection = &packet->Request.connections[i];
+            if (connection->type == XAUTH_FAMILY_INTERNET && connection->address.length == 4)
+            {
+                family = connection->type;
+                xserver_address = g_inet_address_new_from_bytes (connection->address.data, G_SOCKET_FAMILY_IPV4);
+                break;
+            }
+        }
+    }
+
+    /* Decline if haven't got an address we can connect on */
+    if (!xserver_address)
     {
         response = xdmcp_packet_alloc (XDMCP_Decline);
-        if (strcmp (server->priv->authentication_name, "") == 0)
-            response->Decline.status = g_strdup ("Server does not support authentication");
+        response->Decline.status = g_strdup ("No valid address found");
+        response->Decline.authentication_name = g_strdup (packet->Request.authentication_name);
+        response->Decline.authentication_data.data = authentication_data;
+        response->Decline.authentication_data.length = authentication_data_length;
+        send_packet (socket, address, response);
+        xdmcp_packet_free (response);
+        return;
+    }
+  
+    /* Must be using our authentication scheme */
+    if (strcmp (packet->Request.authentication_name, get_authentication_name (server)) != 0)
+    {
+        response = xdmcp_packet_alloc (XDMCP_Decline);
+        if (server->priv->key)
+            response->Decline.status = g_strdup_printf ("Server only supports %s authentication", get_authentication_name (server));
         else
-            response->Decline.status = g_strdup_printf ("Server only supports %s authentication", server->priv->authentication_name);
+            response->Decline.status = g_strdup ("Server does not support authentication");
         response->Decline.authentication_name = g_strdup ("");
         send_packet (socket, address, response);
         xdmcp_packet_free (response);
@@ -293,43 +337,45 @@ handle_request (XDMCPServer *server, GSocket *socket, GSocketAddress *address, X
     }
 
     /* Perform requested authentication */
-    if (strcmp (server->priv->authentication_name, "XDM-AUTHENTICATION-1") == 0)
+    if (server->priv->key)
     {
-        guchar input[8], key[8];
+        guint8 input[8], key[8];
 
         memset (input, 0, 8);
         memcpy (input, packet->Request.authentication_data.data, packet->Request.authentication_data.length > 8 ? 8 : packet->Request.authentication_data.length);
 
         /* Setup key */
-        memset (key, 0, 8);
-        memcpy (key, server->priv->authentication_data, server->priv->authentication_data_length > 8 ? 8 : server->priv->authentication_data_length);
+        decode_key (server->priv->key, key);
 
         /* Decode message from server */
-        authentication_data = g_malloc (sizeof (guchar) * 8);
+        authentication_data = g_malloc (sizeof (guint8) * 8);
         authentication_data_length = 8;
 
         XdmcpUnwrap (input, key, rho.data, authentication_data_length);
         XdmcpIncrementKey (&rho);
         XdmcpWrap (rho.data, key, authentication_data, authentication_data_length);
+
+        authorization_name = g_strdup ("XDM-AUTHORIZATION-1");
     }
+    else
+        authorization_name = g_strdup ("MIT-MAGIC-COOKIE-1");
 
     /* Check if they support our authorization */
     for (j = packet->Request.authorization_names; *j; j++)
     {
-        if (strcmp (*j, server->priv->authorization_name) == 0)
+        if (strcmp (*j, authorization_name) == 0)
         {
              match_authorization = TRUE;
              break;
         }
     }
-  
+
+    /* Decline if don't support out authorization */
     if (!match_authorization)
     {
         response = xdmcp_packet_alloc (XDMCP_Decline);
-        if (strcmp (server->priv->authorization_name, "") == 0)
-            response->Decline.status = g_strdup ("Server does not support authorization");
-        else
-            response->Decline.status = g_strdup_printf ("Server only supports %s authorization", server->priv->authorization_name);
+        response->Decline.status = g_strdup_printf ("Server requires %s authorization", authorization_name);
+        g_free (authorization_name);
         response->Decline.authentication_name = g_strdup (packet->Request.authentication_name);
         response->Decline.authentication_data.data = authentication_data;
         response->Decline.authentication_data.length = authentication_data_length;
@@ -339,25 +385,13 @@ handle_request (XDMCPServer *server, GSocket *socket, GSocketAddress *address, X
     }
 
     /* Perform requested authorization */
-    if (strcmp (server->priv->authorization_name, "MIT-MAGIC-COOKIE-1") == 0)
-    {
-        XAuthorization *auth;
-
-        /* Data is the cookie */
-        auth = xauth_new_cookie ();
-        authorization_data = xauth_copy_authorization_data (auth);
-        authorization_data_length = xauth_get_authorization_data_length (auth);
-        session_authorization_data = xauth_copy_authorization_data (auth);
-        session_authorization_data_length = xauth_get_authorization_data_length (auth);
-    }
-    else if (strcmp (server->priv->authorization_name, "XDM-AUTHORIZATION-1") == 0)
+    if (server->priv->key)
     {
         gint i;
-        guchar key[8], session_key[8];
+        guint8 key[8], session_key[8];
 
         /* Setup key */
-        memset (key, 0, 8);
-        memcpy (key, server->priv->authentication_data, server->priv->authentication_data_length > 8 ? 8 : server->priv->authentication_data_length);
+        decode_key (server->priv->key, key);
 
         /* Generate a private session key */
         // FIXME: Pick a good DES key?
@@ -377,50 +411,57 @@ handle_request (XDMCPServer *server, GSocket *socket, GSocketAddress *address, X
         memcpy (session_authorization_data, rho.data, 8);
         memcpy (session_authorization_data + 8, session_key, 8);
     }
-
-    for (i = 0; i < packet->Request.n_connections; i++)
+    else
     {
-        XDMCPConnection *connection;
+        XAuthority *auth;
 
-        connection = &packet->Request.connections[i];
-        switch (connection->type)
-        {
-        case FamilyInternet:
-            if (connection->address.length == 4)
-                address4 = g_inet_address_new_from_bytes (connection->address.data, G_SOCKET_FAMILY_IPV4);
-            break;
-        case FamilyInternet6:
-            if (connection->address.length == 16)
-                address6 = g_inet_address_new_from_bytes (connection->address.data, G_SOCKET_FAMILY_IPV6);          
-            break;
-        }
-    }
+        /* Data is the cookie */
+        auth = xauth_new_cookie (XAUTH_FAMILY_WILD, NULL, 0, "");
+        authorization_data = xauth_copy_authorization_data (auth);
+        authorization_data_length = xauth_get_authorization_data_length (auth);
+        session_authorization_data = xauth_copy_authorization_data (auth);
+        session_authorization_data_length = xauth_get_authorization_data_length (auth);
 
-    if (!address4) // FIXME: && !address6)
-    {
-        response = xdmcp_packet_alloc (XDMCP_Decline);
-        response->Decline.status = g_strdup ("No valid address found");
-        response->Decline.authentication_name = g_strdup (packet->Request.authentication_name);
-        response->Decline.authentication_data.data = authentication_data;
-        response->Decline.authentication_data.length = authentication_data_length;
-        send_packet (socket, address, response);
-        xdmcp_packet_free (response);
-        return;
+        g_object_unref (auth);
     }
 
     session = add_session (server);
-    session->priv->address = address4;
-    session->priv->address6 = address6;
-    session->priv->authorization_name = g_strdup (server->priv->authorization_name);
-    session->priv->authorization_data = session_authorization_data;
-    session->priv->authorization_data_length = session_authorization_data_length;
+    session->priv->address = xserver_address;
+    session->priv->display_number = packet->Request.display_number;
+    display_number = g_strdup_printf ("%d", packet->Request.display_number);
+
+    /* We need to check if this is the loopback address and set the authority
+     * for a local connection if this is so as XCB treats "127.0.0.1" as local
+     * always */
+    if (g_inet_address_get_is_loopback (xserver_address))
+    {
+        gchar hostname[1024];
+        gethostname (hostname, 1024);
+
+        session->priv->authority = xauth_new (XAUTH_FAMILY_LOCAL,
+                                              (guint8 *) hostname,
+                                              strlen (hostname),
+                                              display_number,
+                                              authorization_name,
+                                              session_authorization_data,
+                                              session_authorization_data_length);
+    }
+    else
+        session->priv->authority = xauth_new (family,
+                                              g_inet_address_to_bytes (G_INET_ADDRESS (xserver_address)),
+                                              g_inet_address_get_native_size (G_INET_ADDRESS (xserver_address)),
+                                              display_number,
+                                              authorization_name,
+                                              session_authorization_data,
+                                              session_authorization_data_length);
+    g_free (display_number);
 
     response = xdmcp_packet_alloc (XDMCP_Accept);
     response->Accept.session_id = xdmcp_session_get_id (session);
     response->Accept.authentication_name = g_strdup (packet->Request.authentication_name);
     response->Accept.authentication_data.data = authentication_data;
     response->Accept.authentication_data.length = authentication_data_length;
-    response->Accept.authorization_name = g_strdup (server->priv->authorization_name);
+    response->Accept.authorization_name = authorization_name;
     response->Accept.authorization_data.data = authorization_data;
     response->Accept.authorization_data.length = authorization_data_length;
     send_packet (socket, address, response);
@@ -431,44 +472,10 @@ static void
 handle_manage (XDMCPServer *server, GSocket *socket, GSocketAddress *address, XDMCPPacket *packet)
 {
     XDMCPSession *session;
+    gboolean result;
 
     session = get_session (server, packet->Manage.session_id);
-    if (session)
-    {
-        gboolean result;
-
-        /* Ignore duplicate requests */
-        if (session->priv->started)
-        {
-            if (session->priv->display_number != packet->Manage.display_number ||
-                strcmp (session->priv->display_class, packet->Manage.display_class) != 0)
-                g_warning ("Duplicate Manage received with different data");
-            return;
-        }
-
-        session->priv->display_number = packet->Manage.display_number;  
-        session->priv->display_class = g_strdup (packet->Manage.display_class);
-
-        g_signal_emit (server, signals[NEW_SESSION], 0, session, &result);
-        if (result)
-        {
-            /* Cancel the inactive timer */
-            g_source_remove (session->priv->inactive_timeout);
-
-            session->priv->started = TRUE;
-        }
-        else
-        {
-            XDMCPPacket *response;
-
-            response = xdmcp_packet_alloc (XDMCP_Failed);
-            response->Failed.session_id = packet->Manage.session_id;
-            response->Failed.status = g_strdup_printf ("Failed to connect to display :%d", packet->Manage.display_number);
-            send_packet (socket, address, response);
-            xdmcp_packet_free (response);
-        }
-    }
-    else
+    if (!session)
     {
         XDMCPPacket *response;
 
@@ -476,6 +483,50 @@ handle_manage (XDMCPServer *server, GSocket *socket, GSocketAddress *address, XD
         response->Refuse.session_id = packet->Manage.session_id;
         send_packet (socket, address, response);
         xdmcp_packet_free (response);
+
+        return;
+    }
+
+    /* Ignore duplicate requests */
+    if (session->priv->started)
+    {
+        if (session->priv->display_number != packet->Manage.display_number ||
+            strcmp (session->priv->display_class, packet->Manage.display_class) != 0)
+            g_debug ("Ignoring duplicate Manage with different data");
+        return;
+    }
+
+    /* Reject if has changed display number */
+    if (packet->Manage.display_number != session->priv->display_number)
+    {
+        XDMCPPacket *response;
+
+        g_debug ("Received Manage for display number %d, but Request was %d", packet->Manage.display_number, session->priv->display_number);
+        response = xdmcp_packet_alloc (XDMCP_Refuse);
+        response->Refuse.session_id = packet->Manage.session_id;
+        send_packet (socket, address, response);
+        xdmcp_packet_free (response);
+    }
+
+    session->priv->display_class = g_strdup (packet->Manage.display_class);
+
+    g_signal_emit (server, signals[NEW_SESSION], 0, session, &result);
+    if (result)
+    {
+        /* Cancel the inactive timer */
+        g_source_remove (session->priv->inactive_timeout);
+
+        session->priv->started = TRUE;
+    }
+    else
+    {
+        XDMCPPacket *response;
+
+        response = xdmcp_packet_alloc (XDMCP_Failed);
+        response->Failed.session_id = packet->Manage.session_id;
+        response->Failed.status = g_strdup_printf ("Failed to connect to display :%d", packet->Manage.display_number);
+        send_packet (socket, address, response);
+        xdmcp_packet_free (response);
     }
 }
 
@@ -488,7 +539,7 @@ handle_keep_alive (XDMCPServer *server, GSocket *socket, GSocketAddress *address
 
     session = get_session (server, packet->KeepAlive.session_id);
     if (session)
-        alive = TRUE; //xdmcp_session_get_alive (session);
+        alive = TRUE; // FIXME: xdmcp_session_get_alive (session);
 
     response = xdmcp_packet_alloc (XDMCP_Alive);
     response->Alive.session_running = alive;
@@ -506,11 +557,15 @@ read_cb (GSocket *socket, GIOCondition condition, XDMCPServer *server)
     gssize n_read;
 
     n_read = g_socket_receive_from (socket, &address, data, 1024, NULL, &error);
+    if (error)
+        g_warning ("Failed to read from XDMCP socket: %s", error->message);
+    g_clear_error (&error);
+
     if (n_read > 0)
     {
         XDMCPPacket *packet;
 
-        packet = xdmcp_packet_decode ((guchar *)data, n_read);
+        packet = xdmcp_packet_decode ((guint8 *)data, n_read);
         if (packet)
         {        
             g_debug ("Got %s", xdmcp_packet_tostring (packet));
@@ -539,10 +594,6 @@ read_cb (GSocket *socket, GIOCondition condition, XDMCPServer *server)
             xdmcp_packet_free (packet);
         }
     }
-    else
-        g_warning ("Failed to read from XDMCP socket: %s", error->message);
-
-    g_clear_error (&error);
 
     return TRUE;
 }
@@ -574,27 +625,32 @@ xdmcp_server_start (XDMCPServer *server)
 {
     GSource *source;
     GError *error = NULL;
+
+    g_return_val_if_fail (server != NULL, FALSE);
   
     server->priv->socket = open_udp_socket (G_SOCKET_FAMILY_IPV4, server->priv->port, &error);
+    if (error)
+        g_warning ("Failed to create IPv4 XDMCP socket: %s", error->message);
+    g_clear_error (&error);
+  
     if (server->priv->socket)
     {
         source = g_socket_create_source (server->priv->socket, G_IO_IN, NULL);
         g_source_set_callback (source, (GSourceFunc) read_cb, server, NULL);
         g_source_attach (source, NULL);
     }
-    else
-        g_warning ("Failed to create IPv4 XDMCP socket: %s", error->message);
+    
+    server->priv->socket6 = open_udp_socket (G_SOCKET_FAMILY_IPV6, server->priv->port, &error);
+    if (error)
+        g_warning ("Failed to create IPv6 XDMCP socket: %s", error->message);
     g_clear_error (&error);
-    server->priv->socket6 = open_udp_socket (G_SOCKET_FAMILY_IPV6, server->priv->port, &error);  
+
     if (server->priv->socket6)
     {
         source = g_socket_create_source (server->priv->socket6, G_IO_IN, NULL);
         g_source_set_callback (source, (GSourceFunc) read_cb, server, NULL);
         g_source_attach (source, NULL);
     }
-    else
-        g_warning ("Failed to create IPv6 XDMCP socket: %s", error->message);
-    g_clear_error (&error);
 
     if (!server->priv->socket && !server->priv->socket6)
         return FALSE;
@@ -611,8 +667,6 @@ xdmcp_server_init (XDMCPServer *server)
     server->priv->hostname = g_strdup ("");
     server->priv->status = g_strdup ("");
     server->priv->sessions = g_hash_table_new_full (g_direct_hash, g_direct_equal, NULL, g_object_unref);
-    server->priv->authentication_name = g_strdup ("");
-    server->priv->authorization_name = g_strdup ("");
 }
 
 static void
@@ -628,10 +682,7 @@ xdmcp_server_finalize (GObject *object)
         g_object_unref (self->priv->socket6);
     g_free (self->priv->hostname);
     g_free (self->priv->status);
-    g_free (self->priv->authentication_name);
-    g_free (self->priv->authentication_data);
-    g_free (self->priv->authorization_name);
-    g_free (self->priv->authorization_data);
+    g_free (self->priv->key);
     g_hash_table_unref (self->priv->sessions);
   
     G_OBJECT_CLASS (xdmcp_server_parent_class)->finalize (object);  
